
# q-expansion of newform 8040.2.a.l, downloaded from the LMFDB on 04 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 8040
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [1], [1], [4], [6], [-2], [0], [-4], [0], [2], [-10], [-8], [8], [4], [12], [10], [-8], [0], [-1], [8], [14], [10], [-6], [-10], [2], [18], [14], [-18], [0], [-6], [-10], [4], [-18], [-8], [6], [-8], [-8], [16], [-12], [-2], [2], [14], [-20], [22], [-6], [4], [-4], [6], [-18], [12], [-6], [0], [-14], [10], [0], [20], [-10], [14], [-16], [0], [8], [-14], [32], [-8], [6], [-30], [-12], [34], [4], [-18], [-2], [8], [0], [-26], [-8], [-16], [-10], [12], [-24], [-30], [16], [18], [32], [14], [-8], [0], [18], [-22], [18], [8], [26], [16], [8], [20], [8], [-24], [14], [12], [-44], [0], [28], [-2], [-36], [-42], [-12], [38], [-24], [34], [16], [-10], [22], [-24], [-36], [-36], [18], [-40], [0], [-24], [-46], [0], [-12], [-38], [42], [-24], [-36], [2], [-34], [-24], [-8], [30], [40], [12], [-40], [-42], [42], [46], [18], [-28], [30], [-4], [-20], [30], [10], [-18], [26], [24], [-44], [-10], [20], [-24], [-32], [-42], [-20], [56], [-16], [40], [10], [-20], [-58], [54], [-30], [-24], [50], [40], [0], [16], [-22], [-24], [-34], [-10], [-36], [18], [0], [58], [10], [12], [40], [-58], [44], [14], [50], [24], [-36], [-36], [24], [34], [2], [-4], [-38], [12], [-26], [-42], [44], [30], [36], [48], [2], [-26], [-22], [-32], [10], [-28], [-34], [38], [-50], [-38], [52], [-54], [42], [-32], [-26], [-18], [-20], [-32], [-60], [-18], [-50], [-38], [-24], [6], [48], [12], [60], [10], [72], [-26], [4], [16], [46], [-2], [-46], [0], [64], [-64], [4], [32], [14], [70], [36], [16], [-42], [64], [52], [24], [-2], [-20], [14], [-6], [4], [-16], [-20], [-42], [-40], [-58], [-58], [-30], [-52], [-32], [2], [22], [-10], [-66], [48], [2], [60], [20], [30], [56], [4], [-58], [64], [60], [-46], [76], [38], [-40], [-12], [-22], [70], [2], [0], [-16], [-16], [36], [44], [-16], [82], [-50], [70], [60], [-30], [-8], [4], [-12], [-86], [-34], [-38], [30], [-60], [52], [-38], [-66], [-36], [-42], [4], [-18], [-74], [26], [48], [38], [-40], [22], [62], [32], [-32], [-2], [0], [-48], [30], [-68], [-4], [-26], [-30], [-32], [-80], [-62], [-4], [80], [-24], [6], [-66], [-2], [14], [-76], [20], [42], [-16], [-60], [38], [6], [-36], [-68], [82], [8], [74], [-10], [-32], [-28], [16], [-18], [-76], [-58], [50], [80], [-48], [-58], [-64], [-38], [18], [72], [-12], [66], [-8], [-40], [-68], [-26], [-70], [-70], [22], [-42], [52], [12], [82], [70], [98], [-52], [36], [2], [60], [-70], [-82], [90], [80], [-54], [-48], [-64], [46], [-60], [-20], [88], [-34], [58], [-4], [-44], [0], [-14], [-56], [-70], [40], [-98], [60], [24], [48], [70], [-38], [18], [18], [2], [16], [-54], [-22], [-50], [10], [12], [-62], [-14], [-24], [-26], [-40], [28], [94], [-44], [-32], [62], [-58], [-46], [-58], [-90], [4], [48], [2], [62], [-60], [16], [100], [102], [14], [80], [-44], [-32], [54], [36], [-88], [72], [14], [-80], [48], [-72], [62], [20], [-20], [56], [-78], [14], [46], [-90], [-20], [84], [-92], [6], [-108]]
